@@ -1,12 +1,12 @@
 //! Frontend message serialization.
 #![allow(missing_docs)]
 
-use byteorder::{WriteBytesExt, BigEndian, ByteOrder};
+use byteorder::{BigEndian, ByteOrder, WriteBytesExt};
 use std::error::Error;
 use std::io;
 use std::marker;
 
-use {Oid, FromUsize, IsNull, write_nullable};
+use crate::{write_nullable, FromUsize, IsNull, Oid};
 
 pub enum Message<'a> {
     Bind {
@@ -16,24 +16,51 @@ pub enum Message<'a> {
         values: &'a [Option<Vec<u8>>],
         result_formats: &'a [i16],
     },
-    CancelRequest { process_id: i32, secret_key: i32 },
-    Close { variant: u8, name: &'a str },
-    CopyData { data: &'a [u8] },
+    CancelRequest {
+        process_id: i32,
+        secret_key: i32,
+    },
+    Close {
+        variant: u8,
+        name: &'a str,
+    },
+    CopyData {
+        data: &'a [u8],
+    },
     CopyDone,
-    CopyFail { message: &'a str },
-    Describe { variant: u8, name: &'a str },
-    Execute { portal: &'a str, max_rows: i32 },
+    CopyFail {
+        message: &'a str,
+    },
+    Describe {
+        variant: u8,
+        name: &'a str,
+    },
+    Execute {
+        portal: &'a str,
+        max_rows: i32,
+    },
     Parse {
         name: &'a str,
         query: &'a str,
         param_types: &'a [Oid],
     },
-    PasswordMessage { password: &'a str },
-    Query { query: &'a str },
-    SaslInitialResponse { mechanism: &'a str, data: &'a [u8] },
-    SaslResponse { data: &'a [u8] },
+    PasswordMessage {
+        password: &'a [u8],
+    },
+    Query {
+        query: &'a str,
+    },
+    SaslInitialResponse {
+        mechanism: &'a str,
+        data: &'a [u8],
+    },
+    SaslResponse {
+        data: &'a [u8],
+    },
     SslRequest,
-    StartupMessage { parameters: &'a [(String, String)] },
+    StartupMessage {
+        parameters: &'a [(String, String)],
+    },
     Sync,
     Terminate,
     #[doc(hidden)]
@@ -75,10 +102,16 @@ impl<'a> Message<'a> {
             Message::CancelRequest {
                 process_id,
                 secret_key,
-            } => Ok(cancel_request(process_id, secret_key, buf)),
+            } => {
+                cancel_request(process_id, secret_key, buf);
+                Ok(())
+            }
             Message::Close { variant, name } => close(variant, name, buf),
             Message::CopyData { data } => copy_data(data, buf),
-            Message::CopyDone => Ok(copy_done(buf)),
+            Message::CopyDone => {
+                copy_done(buf);
+                Ok(())
+            }
             Message::CopyFail { message } => copy_fail(message, buf),
             Message::Describe { variant, name } => describe(variant, name, buf),
             Message::Execute { portal, max_rows } => execute(portal, max_rows, buf),
@@ -93,12 +126,21 @@ impl<'a> Message<'a> {
                 sasl_initial_response(mechanism, data, buf)
             }
             Message::SaslResponse { data } => sasl_response(data, buf),
-            Message::SslRequest => Ok(ssl_request(buf)),
+            Message::SslRequest => {
+                ssl_request(buf);
+                Ok(())
+            }
             Message::StartupMessage { parameters } => {
                 startup_message(parameters.iter().map(|&(ref k, ref v)| (&**k, &**v)), buf)
             }
-            Message::Sync => Ok(sync(buf)),
-            Message::Terminate => Ok(terminate(buf)),
+            Message::Sync => {
+                sync(buf);
+                Ok(())
+            }
+            Message::Terminate => {
+                terminate(buf);
+                Ok(())
+            }
             Message::__ForExtensibility => unreachable!(),
         }
     }
@@ -121,13 +163,13 @@ where
 }
 
 pub enum BindError {
-    Conversion(Box<Error + marker::Sync + Send>),
+    Conversion(Box<dyn Error + marker::Sync + Send>),
     Serialization(io::Error),
 }
 
-impl From<Box<Error + marker::Sync + Send>> for BindError {
+impl From<Box<dyn Error + marker::Sync + Send>> for BindError {
     #[inline]
-    fn from(e: Box<Error + marker::Sync + Send>) -> BindError {
+    fn from(e: Box<dyn Error + marker::Sync + Send>) -> BindError {
         BindError::Conversion(e)
     }
 }
@@ -152,14 +194,14 @@ pub fn bind<I, J, F, T, K>(
 where
     I: IntoIterator<Item = i16>,
     J: IntoIterator<Item = T>,
-    F: FnMut(T, &mut Vec<u8>) -> Result<IsNull, Box<Error + marker::Sync + Send>>,
+    F: FnMut(T, &mut Vec<u8>) -> Result<IsNull, Box<dyn Error + marker::Sync + Send>>,
     K: IntoIterator<Item = i16>,
 {
     buf.push(b'B');
 
     write_body(buf, |buf| {
-        buf.write_cstr(portal)?;
-        buf.write_cstr(statement)?;
+        buf.write_cstr(portal.as_bytes())?;
+        buf.write_cstr(statement.as_bytes())?;
         write_counted(formats, |f, buf| buf.write_i16::<BigEndian>(f), buf)?;
         write_counted(
             values,
@@ -195,10 +237,11 @@ where
 #[inline]
 pub fn cancel_request(process_id: i32, secret_key: i32, buf: &mut Vec<u8>) {
     write_body(buf, |buf| {
-        buf.write_i32::<BigEndian>(80877102).unwrap();
+        buf.write_i32::<BigEndian>(80_877_102).unwrap();
         buf.write_i32::<BigEndian>(process_id).unwrap();
         buf.write_i32::<BigEndian>(secret_key)
-    }).unwrap();
+    })
+    .unwrap();
 }
 
 #[inline]
@@ -206,7 +249,7 @@ pub fn close(variant: u8, name: &str, buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'C');
     write_body(buf, |buf| {
         buf.push(variant);
-        buf.write_cstr(name)
+        buf.write_cstr(name.as_bytes())
     })
 }
 
@@ -229,7 +272,7 @@ pub fn copy_done(buf: &mut Vec<u8>) {
 #[inline]
 pub fn copy_fail(message: &str, buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'f');
-    write_body(buf, |buf| buf.write_cstr(message))
+    write_body(buf, |buf| buf.write_cstr(message.as_bytes()))
 }
 
 #[inline]
@@ -237,7 +280,7 @@ pub fn describe(variant: u8, name: &str, buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'D');
     write_body(buf, |buf| {
         buf.push(variant);
-        buf.write_cstr(name)
+        buf.write_cstr(name.as_bytes())
     })
 }
 
@@ -245,7 +288,7 @@ pub fn describe(variant: u8, name: &str, buf: &mut Vec<u8>) -> io::Result<()> {
 pub fn execute(portal: &str, max_rows: i32, buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'E');
     write_body(buf, |buf| {
-        buf.write_cstr(portal)?;
+        buf.write_cstr(portal.as_bytes())?;
         buf.write_i32::<BigEndian>(max_rows).unwrap();
         Ok(())
     })
@@ -258,15 +301,15 @@ where
 {
     buf.push(b'P');
     write_body(buf, |buf| {
-        buf.write_cstr(name)?;
-        buf.write_cstr(query)?;
+        buf.write_cstr(name.as_bytes())?;
+        buf.write_cstr(query.as_bytes())?;
         write_counted(param_types, |t, buf| buf.write_u32::<BigEndian>(t), buf)?;
         Ok(())
     })
 }
 
 #[inline]
-pub fn password_message(password: &str, buf: &mut Vec<u8>) -> io::Result<()> {
+pub fn password_message(password: &[u8], buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'p');
     write_body(buf, |buf| buf.write_cstr(password))
 }
@@ -274,14 +317,14 @@ pub fn password_message(password: &str, buf: &mut Vec<u8>) -> io::Result<()> {
 #[inline]
 pub fn query(query: &str, buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'Q');
-    write_body(buf, |buf| buf.write_cstr(query))
+    write_body(buf, |buf| buf.write_cstr(query.as_bytes()))
 }
 
 #[inline]
 pub fn sasl_initial_response(mechanism: &str, data: &[u8], buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'p');
     write_body(buf, |buf| {
-        buf.write_cstr(mechanism)?;
+        buf.write_cstr(mechanism.as_bytes())?;
         let len = i32::from_usize(data.len())?;
         buf.write_i32::<BigEndian>(len)?;
         buf.extend_from_slice(data);
@@ -292,12 +335,15 @@ pub fn sasl_initial_response(mechanism: &str, data: &[u8], buf: &mut Vec<u8>) ->
 #[inline]
 pub fn sasl_response(data: &[u8], buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'p');
-    write_body(buf, |buf| Ok(buf.extend_from_slice(data)))
+    write_body(buf, |buf| {
+        buf.extend_from_slice(data);
+        Ok(())
+    })
 }
 
 #[inline]
 pub fn ssl_request(buf: &mut Vec<u8>) {
-    write_body(buf, |buf| buf.write_i32::<BigEndian>(80877103)).unwrap();
+    write_body(buf, |buf| buf.write_i32::<BigEndian>(80_877_103)).unwrap();
 }
 
 #[inline]
@@ -306,10 +352,10 @@ where
     I: IntoIterator<Item = (&'a str, &'a str)>,
 {
     write_body(buf, |buf| {
-        buf.write_i32::<BigEndian>(196608).unwrap();
+        buf.write_i32::<BigEndian>(196_608).unwrap();
         for (key, value) in parameters {
-            buf.write_cstr(key.as_ref())?;
-            buf.write_cstr(value.as_ref())?;
+            buf.write_cstr(key.as_bytes())?;
+            buf.write_cstr(value.as_bytes())?;
         }
         buf.push(0);
         Ok(())
@@ -329,19 +375,19 @@ pub fn terminate(buf: &mut Vec<u8>) {
 }
 
 trait WriteCStr {
-    fn write_cstr(&mut self, s: &str) -> Result<(), io::Error>;
+    fn write_cstr(&mut self, s: &[u8]) -> Result<(), io::Error>;
 }
 
 impl WriteCStr for Vec<u8> {
     #[inline]
-    fn write_cstr(&mut self, s: &str) -> Result<(), io::Error> {
-        if s.as_bytes().contains(&0) {
+    fn write_cstr(&mut self, s: &[u8]) -> Result<(), io::Error> {
+        if s.contains(&0) {
             return Err(io::Error::new(
                 io::ErrorKind::InvalidInput,
                 "string contains embedded null",
             ));
         }
-        self.extend_from_slice(s.as_bytes());
+        self.extend_from_slice(s);
         self.push(0);
         Ok(())
     }
